==> fap-0.9.0.tar.bz2: fap-0.9.0/src/audio/input/ffmpegBackend.cpp <==
/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ffmpegBackend.h"

#ifdef HAVE_FFMPEG

#include "loadsym.h"

#define CREDITS		"ffmpeg"
#define LINK		"http://www.ffmpeg.org/"

const char ffmpegBackend::name[]="Ffmpeg";

#ifdef _WIN32
# define AVFORMATLIB	"avformat-52.dll"
# define AVCODECLIB	"avcodec-52.dll"
#else
# define AVFORMATLIB	"libavformat.so.52"
# define AVCODECLIB	"libavcodec.so.52"
#endif

const AutoDLL ffmpegBackend::_avformat(AVFORMATLIB);
const AutoDLL ffmpegBackend::_avcodec(AVCODECLIB);

int (*ffmpegBackend::dl_av_open_input_file)(AVFormatContext**, const char*, AVInputFormat*, int, AVFormatParameters*)=0;
void (*ffmpegBackend::dl_av_close_input_file)(AVFormatContext*)=0;
int (*ffmpegBackend::dl_av_find_stream_info)(AVFormatContext*)=0;
int (*ffmpegBackend::dl_av_read_frame)(AVFormatContext*, AVPacket*)=0;
int (*ffmpegBackend::dl_av_seek_frame)(AVFormatContext*, int, int64_t, int)=0;
AVMetadataTag* (*ffmpegBackend::dl_av_metadata_get)(AVMetadata*, const char*, const AVMetadataTag*, int)=0;
AVCodec* (*ffmpegBackend::dl_avcodec_find_decoder)(enum CodecID)=0;
int (*ffmpegBackend::dl_avcodec_open)(AVCodecContext*, AVCodec*)=0;
int (*ffmpegBackend::dl_avcodec_decode_audio3)(AVCodecContext*, int16_t*, int*, AVPacket*)=0;
void (*ffmpegBackend::dl_av_init_packet)(AVPacket*)=0;
void (*ffmpegBackend::dl_avcodec_flush_buffers)(AVCodecContext*)=0;
int (*ffmpegBackend::dl_avcodec_close)(AVCodecContext*)=0;
void (*ffmpegBackend::dl_av_free_packet)(AVPacket*)=0;
// int64_t (*ffmpegBackend::dl_av_rescale_q)(int64_t, AVRational, AVRational)=0;

FXString ffmpegBackend::_ext="*.(";

// Based on code from GNUsound 0.7.4 - Copyright (C) 2002-2004, Pascal Haakmat
// http://www.gnu.org/software/gnusound/
// and on info from Martin Böhme's article "Using libavformat and libavcodec"
// http://www.inb.uni-luebeck.de/~boehme/using_libavcodec.html

/*****************************************************************/

size_t ffmpegBackend::fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds) {

	size_t n=_decodeBufOffset;
	FXuchar* buf=_decodeBuf;

	while (n<=bufferSize) {
		while (!_packet.data) {
			if (dl_av_read_frame(_formatContext, &_packet)<0) {
				FXTRACE((10, "Last frame: %d\n", n));
				memcpy(buffer, buf, n);
				_decodeBufOffset=0;
				return n;
			}
			if (_packet.stream_index!=_audioStreamIndex) {
				dl_av_free_packet(&_packet);
				_packet.data=0;
			} else
				_packetOffset=0;
		}
		int frame_size=AVCODEC_MAX_AUDIO_FRAME_SIZE*2-n;

		AVPacket avpkt;
		dl_av_init_packet(&avpkt);
		avpkt.data=_packet.data+_packetOffset;
		avpkt.size=_packet.size-_packetOffset;
		const int used=dl_avcodec_decode_audio3(_audioStream->codec, (FXshort*)(buf+n), &frame_size, &avpkt);

		FXTRACE((50, "frame_size: %d\n", frame_size));
		if (used>=0) {
			n+=frame_size;
			_packetOffset+=used;
		}
		if ((used<0) || (_packet.size<=_packetOffset)) {
			dl_av_free_packet(&_packet);
			_packet.data=0;
		}
	}

	memcpy(buffer, buf, bufferSize);
	_decodeBufOffset=n-bufferSize;
	memcpy(buf, buf+bufferSize, _decodeBufOffset);

	return bufferSize;
}

/*****************************************************************/

bool ffmpegBackend::init() {

	if (!_avformat.loaded() || !_avcodec.loaded())
		return false;


	LOADSYM(_avcodec, avcodec_decode_audio3, int(*)(AVCodecContext*, int16_t*, int*, AVPacket*))
	LOADSYM(_avcodec, av_init_packet, void(*)(AVPacket*));

	LOADSYM(_avformat, av_open_input_file, int(*)(AVFormatContext**, const char*, AVInputFormat*, int, AVFormatParameters*))
	LOADSYM(_avformat, av_close_input_file, void(*)(AVFormatContext*))
	LOADSYM(_avformat, av_find_stream_info, int(*)(AVFormatContext*))
	LOADSYM(_avformat, av_read_frame, int(*)(AVFormatContext*, AVPacket*))
	LOADSYM(_avformat, av_seek_frame, int(*)(AVFormatContext*, int, int64_t, int))
	LOADSYM(_avformat, av_metadata_get, AVMetadataTag*(*)(AVMetadata*, const char*, const AVMetadataTag*, int))
	LOADSYM(_avcodec, avcodec_find_decoder, AVCodec*(*)(enum CodecID))
	LOADSYM(_avcodec, avcodec_open, int(*)(AVCodecContext*, AVCodec*))
	LOADSYM(_avcodec, avcodec_flush_buffers, void(*)(AVCodecContext*))
	LOADSYM(_avcodec, avcodec_close, int(*)(AVCodecContext*))
	LOADSYM(_avcodec, av_free_packet, void(*)(AVPacket*))
	//LOADSYM(_avcodec, av_rescale_q, int64_t (*)(int64_t, AVRational, AVRational)) // FIXME in libavutil now?

	AVInputFormat *(*dl_av_find_input_format)(const char*);
	LOADSYM(_avformat, av_find_input_format, AVInputFormat*(*)(const char*))

	void (*dl_av_register_all)(void);
	LOADSYM(_avformat, av_register_all, void(*)(void))

	dl_av_register_all();

	// register supported extensions
	if (dl_av_find_input_format("aac"))
		_ext.append("aac|");
	if (dl_av_find_input_format("ape"))
		_ext.append("ape|apl|mac|");
	if (dl_av_find_input_format("asf"))
		_ext.append("wma|");
	if (dl_av_find_input_format("au"))
		_ext.append("au|");
	if (dl_av_find_input_format("flac"))
		_ext.append("flac|");
	if (dl_av_find_input_format("mp3"))
		_ext.append("mp2|mp3|m2a|");
	if (dl_av_find_input_format("mpc"))
		_ext.append("mpc|");
	if (dl_av_find_input_format("ogg"))
		_ext.append("ogg|");
	if (dl_av_find_input_format("rm"))
		_ext.append("rm|ra|ram|");
	if (dl_av_find_input_format("tta"))
		_ext.append("tta|");
	if (dl_av_find_input_format("voc"))
		_ext.append("voc|");
	if (dl_av_find_input_format("wav"))
		_ext.append("wav|");
	if (dl_av_find_input_format("wv"))
		_ext.append("wv|");

	_ext.replace(_ext.length()-1, ')');

	FXTRACE((10,"Ffmpeg extensions supported: %s\n", _ext.text()));
	return true;
}

bool ffmpegBackend::supports(const char* fileName) {
#if FOX_MINOR>6
	return FXPath::match(FXPath::name(fileName), _ext, NoEscape|PathName|CaseFold);
#else
	return FXPath::match(_ext, FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD);
#endif
}

inline FXString ffmpegBackend::ext() const { return _ext; }

ffmpegBackend::ffmpegBackend() :
	inputBackend(name),
	_audioStream(0),
	_formatContext(0) {}

ffmpegBackend::~ffmpegBackend() {

	close();
}

const char* ffmpegBackend::getMetadata(const char* type) {

	AVMetadataTag* mTag=0;
	FXString info;
	for (;;) {
		mTag=dl_av_metadata_get(_formatContext->metadata, type, mTag, 0);
		if (!mTag)
			break;
		if (!info.empty())
			info.append(", ");
		info.append(FXString(mTag->value).trim());
	}

	return info.text();
}

bool ffmpegBackend::open(const FXString& fileName) {

	close();

	AVFormatContext *fc;

	if (dl_av_open_input_file(&fc, fileName.text(), 0, 0, 0))
		return false;

	int audioStream=-1;

	if (dl_av_find_stream_info(fc)<0)
		goto error;

	for (FXuint i=0; i<fc->nb_streams; i++) {
		if (fc->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO) {
			audioStream=i;
			break;
		}
	}

	if (!openStream(fc, audioStream))
		goto error;

	_formatContext=fc;
	_audioStreamIndex=audioStream;
	_audioStream=fc->streams[audioStream];
	_decodeBufOffset=0;
	_packet.data=0;

	_metaData.addInfo(metaData::TITLE, getMetadata("title"));
	_metaData.addInfo(metaData::ARTIST, getMetadata("artist"));
	_metaData.addInfo(metaData::ALBUM, getMetadata("album"));
	_metaData.addInfo(metaData::GENRE, getMetadata("genre"));
	_metaData.addInfo("Copyright", getMetadata("copyright"));
	_metaData.addInfo(metaData::YEAR, getMetadata("date"));
	_metaData.addInfo(metaData::TRACK, getMetadata("track"));
	_metaData.addInfo(metaData::COMMENT, getMetadata("comment"));

	time(fc->duration/1000000);

	songLoaded(fileName.text());
	return true;

error:
	dl_av_close_input_file(fc);
	return false;
}

void ffmpegBackend::close() {

	if (!songLoaded())
		return;

	if (_packet.data)
		dl_av_free_packet(&_packet);

	dl_avcodec_close(_audioStream->codec);
	dl_av_close_input_file(_formatContext);

	_audioStream=0;
	_formatContext=0;

	songLoaded(0);
}

bool ffmpegBackend::seek(const int pos) {

	if (dl_av_seek_frame(_formatContext,
		_audioStreamIndex,
		//dl_av_rescale_q(pos, AV_TIME_BASE_Q, _audioStream->time_base),
		0,
		AVSEEK_FLAG_ANY)<0)
		return false;

	_decodeBufOffset=0;
	if (_packet.data) {
		dl_av_free_packet(&_packet);
		_packet.data=0;
	}

	dl_avcodec_flush_buffers(_formatContext->streams[_audioStreamIndex]->codec);

	return true;
}

bool ffmpegBackend::openStream(AVFormatContext* fc, const int streamIndex) {

	if ((streamIndex<0) || (streamIndex>=fc->nb_streams))
		return false;

	AVCodecContext* dec=fc->streams[streamIndex]->codec;
	AVCodec* codec=dl_avcodec_find_decoder(dec->codec_id);
	if (!codec)
		return false;

	if (dl_avcodec_open(dec, codec)<0)
		return false;

	return true;
}

/*****************************************************************/

FXIMPLEMENT(ffmpegConfig, configFrame, 0, 0)

ffmpegConfig::ffmpegConfig(FXPacker* win) :
	configFrame(win, ffmpegBackend::name, CREDITS, LINK) {

	new FXLabel(matrix(), tr("No settings available"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
}

#endif

==> fap-0.9.2.tar.bz2: fap-0.9.2/src/audio/input/ffmpegBackend.cpp <==
/*
 *  Copyright (C) 2006-2012 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ffmpegBackend.h"

#ifdef HAVE_FFMPEG

#include "loadsym.h"

#define CREDITS		"ffmpeg"
#define LINK		"http://www.ffmpeg.org/"

const char ffmpegBackend::name[]="Ffmpeg";

#ifdef _WIN32
# define AVFORMATLIB	"avformat-" AV_STRINGIFY(LIBAVFORMAT_VERSION_MAJOR) ".dll"
# define AVCODECLIB	"avcodec-" AV_STRINGIFY(LIBAVCODEC_VERSION_MAJOR) ".dll"
# define AVUTILLIB	"avutil-" AV_STRINGIFY(LIBAVUTIL_VERSION_MAJOR) ".dll"
#else
# define AVFORMATLIB	"libavformat.so." AV_STRINGIFY(LIBAVFORMAT_VERSION_MAJOR)
# define AVCODECLIB	"libavcodec.so." AV_STRINGIFY(LIBAVCODEC_VERSION_MAJOR)
# define AVUTILLIB	"libavutil.so." AV_STRINGIFY(LIBAVUTIL_VERSION_MAJOR)
#endif

const AutoDLL ffmpegBackend::_avformat(AVFORMATLIB);
const AutoDLL ffmpegBackend::_avcodec(AVCODECLIB);
const AutoDLL ffmpegBackend::_avutil(AVUTILLIB);

int (*ffmpegBackend::dl_av_open_input_file)(AVFormatContext**, const char*, AVInputFormat*, int, AVFormatParameters*)=0;
void (*ffmpegBackend::dl_av_close_input_file)(AVFormatContext*)=0;
int (*ffmpegBackend::dl_av_find_stream_info)(AVFormatContext*)=0;
int (*ffmpegBackend::dl_av_read_frame)(AVFormatContext*, AVPacket*)=0;
int (*ffmpegBackend::dl_av_seek_frame)(AVFormatContext*, int, int64_t, int)=0;
AVDictionaryEntry* (*ffmpegBackend::dl_av_dict_get)(AVDictionary*, const char*, const AVDictionaryEntry*, int)=0;
AVCodec* (*ffmpegBackend::dl_avcodec_find_decoder)(enum CodecID)=0;
int (*ffmpegBackend::dl_avcodec_open)(AVCodecContext*, AVCodec*)=0;
int (*ffmpegBackend::dl_avcodec_decode_audio3)(AVCodecContext*, int16_t*, int*, AVPacket*)=0;
void (*ffmpegBackend::dl_av_init_packet)(AVPacket*)=0;
void (*ffmpegBackend::dl_avcodec_flush_buffers)(AVCodecContext*)=0;
int (*ffmpegBackend::dl_avcodec_close)(AVCodecContext*)=0;
void (*ffmpegBackend::dl_av_free_packet)(AVPacket*)=0;
// int64_t (*ffmpegBackend::dl_av_rescale_q)(int64_t, AVRational, AVRational)=0;

FXString ffmpegBackend::_ext="*.(";

// Based on code from GNUsound 0.7.4 - Copyright (C) 2002-2004, Pascal Haakmat
// http://www.gnu.org/software/gnusound/
// and on info from Martin Böhme's article "Using libavformat and libavcodec"
// http://www.inb.uni-luebeck.de/~boehme/using_libavcodec.html

/*****************************************************************/

size_t ffmpegBackend::fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds) {

	size_t n=_decodeBufOffset;
	FXuchar* buf=_decodeBuf;

	while (n<=bufferSize) {
		while (!_packet.data) {
			if (dl_av_read_frame(_formatContext, &_packet)<0) {
				FXTRACE((10, "Last frame: %d\n", n));
				memcpy(buffer, buf, n);
				_decodeBufOffset=0;
				return n;
			}
			if (_packet.stream_index!=_audioStreamIndex) {
				dl_av_free_packet(&_packet);
				_packet.data=0;
			} else
				_packetOffset=0;
		}
		int frame_size=AVCODEC_MAX_AUDIO_FRAME_SIZE*2-n;

		AVPacket avpkt;
		dl_av_init_packet(&avpkt);
		avpkt.data=_packet.data+_packetOffset;
		avpkt.size=_packet.size-_packetOffset;
		const int used=dl_avcodec_decode_audio3(_audioStream->codec, (FXshort*)(buf+n), &frame_size, &avpkt);

		FXTRACE((50, "frame_size: %d\n", frame_size));
		if (used>=0) {
			n+=frame_size;
			_packetOffset+=used;
		}
		if ((used<0) || (_packet.size<=_packetOffset)) {
			dl_av_free_packet(&_packet);
			_packet.data=0;
		}
	}

	memcpy(buffer, buf, bufferSize);
	_decodeBufOffset=n-bufferSize;
	memcpy(buf, buf+bufferSize, _decodeBufOffset);

	return bufferSize;
}

/*****************************************************************/

bool ffmpegBackend::init() {

	if (!_avformat.loaded() || !_avcodec.loaded() || !_avutil.loaded())
		return false;


	LOADSYM(_avcodec, avcodec_decode_audio3, int(*)(AVCodecContext*, int16_t*, int*, AVPacket*))
	LOADSYM(_avcodec, av_init_packet, void(*)(AVPacket*));

	LOADSYM(_avformat, av_open_input_file, int(*)(AVFormatContext**, const char*, AVInputFormat*, int, AVFormatParameters*))
	LOADSYM(_avformat, av_close_input_file, void(*)(AVFormatContext*))
	LOADSYM(_avformat, av_find_stream_info, int(*)(AVFormatContext*))
	LOADSYM(_avformat, av_read_frame, int(*)(AVFormatContext*, AVPacket*))
	LOADSYM(_avformat, av_seek_frame, int(*)(AVFormatContext*, int, int64_t, int))
	LOADSYM(_avutil, av_dict_get, AVDictionaryEntry*(*)(AVDictionary*, const char*, const AVDictionaryEntry*, int))
	LOADSYM(_avcodec, avcodec_find_decoder, AVCodec*(*)(enum CodecID))
	LOADSYM(_avcodec, avcodec_open, int(*)(AVCodecContext*, AVCodec*))
	LOADSYM(_avcodec, avcodec_flush_buffers, void(*)(AVCodecContext*))
	LOADSYM(_avcodec, avcodec_close, int(*)(AVCodecContext*))
	LOADSYM(_avcodec, av_free_packet, void(*)(AVPacket*))
	//LOADSYM(_avcodec, av_rescale_q, int64_t (*)(int64_t, AVRational, AVRational)) // FIXME in libavutil now?

	AVInputFormat *(*dl_av_find_input_format)(const char*);
	LOADSYM(_avformat, av_find_input_format, AVInputFormat*(*)(const char*))

	void (*dl_av_register_all)(void);
	LOADSYM(_avformat, av_register_all, void(*)(void))

	dl_av_register_all();

	// register supported extensions
	if (dl_av_find_input_format("aac"))
		_ext.append("aac|");
	if (dl_av_find_input_format("ape"))
		_ext.append("ape|apl|mac|");
	if (dl_av_find_input_format("asf"))
		_ext.append("wma|");
	if (dl_av_find_input_format("au"))
		_ext.append("au|");
	if (dl_av_find_input_format("flac"))
		_ext.append("flac|");
	if (dl_av_find_input_format("mp3"))
		_ext.append("mp2|mp3|m2a|");
	if (dl_av_find_input_format("mpc"))
		_ext.append("mpc|");
	if (dl_av_find_input_format("ogg"))
		_ext.append("ogg|");
	if (dl_av_find_input_format("rm"))
		_ext.append("rm|ra|ram|");
	if (dl_av_find_input_format("tta"))
		_ext.append("tta|");
	if (dl_av_find_input_format("voc"))
		_ext.append("voc|");
	if (dl_av_find_input_format("wav"))
		_ext.append("wav|");
	if (dl_av_find_input_format("wv"))
		_ext.append("wv|");

	_ext.replace(_ext.length()-1, ')');

	FXTRACE((10,"Ffmpeg extensions supported: %s\n", _ext.text()));
	return true;
}

bool ffmpegBackend::supports(const char* fileName) {
#if FOX_MINOR>6
	return FXPath::match(FXPath::name(fileName), _ext, NoEscape|PathName|CaseFold);
#else
	return FXPath::match(_ext, FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD);
#endif
}

inline FXString ffmpegBackend::ext() const { return _ext; }

ffmpegBackend::ffmpegBackend() :
	inputBackend(name),
	_audioStream(0),
	_formatContext(0) {}

ffmpegBackend::~ffmpegBackend() {

	close();
}

const char* ffmpegBackend::getMetadata(const char* type) {

	AVDictionaryEntry* mTag=0;
	FXString info;
	for (;;) {
		mTag=dl_av_dict_get(_formatContext->metadata, type, mTag, 0);
		if (!mTag)
			break;
		if (!info.empty())
			info.append(", ");
		info.append(FXString(mTag->value).trim());
	}

	return info.text();
}

bool ffmpegBackend::open(const FXString& fileName) {

	close();

	AVFormatContext *fc;

	if (dl_av_open_input_file(&fc, fileName.text(), 0, 0, 0))
		return false;

	int audioStream=-1;

	if (dl_av_find_stream_info(fc)<0)
		goto error;

	for (FXuint i=0; i<fc->nb_streams; i++) {
#if LIBAVCODEC_VERSION_MAJOR < 53
		if (fc->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO) {
#else
		if (fc->streams[i]->codec->codec_type==AVMEDIA_TYPE_AUDIO) {
#endif
			audioStream=i;
			break;
		}
	}

	if (!openStream(fc, audioStream))
		goto error;

	_formatContext=fc;
	_audioStreamIndex=audioStream;
	_audioStream=fc->streams[audioStream];
	_decodeBufOffset=0;
	_packet.data=0;

	switch(_audioStream->codec->sample_fmt) {
	case AV_SAMPLE_FMT_U8:
		_precision=U8;
		break;
	case AV_SAMPLE_FMT_S16:
		_precision=S16;
		break;
	case AV_SAMPLE_FMT_S32:
		_precision=S32;
		break;
	case AV_SAMPLE_FMT_FLT:
		_precision=SAMPLE_FLOAT;
		break;
	default:
		goto error;
	}

	_metaData.addInfo(metaData::TITLE, getMetadata("title"));
	_metaData.addInfo(metaData::ARTIST, getMetadata("artist"));
	_metaData.addInfo(metaData::ALBUM, getMetadata("album"));
	_metaData.addInfo(metaData::GENRE, getMetadata("genre"));
	_metaData.addInfo(gettext("copyright"), getMetadata("copyright"));
	_metaData.addInfo(metaData::YEAR, getMetadata("date"));
	_metaData.addInfo(metaData::TRACK, getMetadata("track"));
	_metaData.addInfo(metaData::COMMENT, getMetadata("comment"));

	time(fc->duration/1000000);

	songLoaded(fileName.text());
	return true;

error:
	dl_av_close_input_file(fc);
	return false;
}

void ffmpegBackend::close() {

	if (!songLoaded())
		return;

	if (_packet.data)
		dl_av_free_packet(&_packet);

	dl_avcodec_close(_audioStream->codec);
	dl_av_close_input_file(_formatContext);

	_audioStream=0;
	_formatContext=0;

	songLoaded(0);
}

bool ffmpegBackend::seek(const int pos) {

	if (dl_av_seek_frame(_formatContext,
		_audioStreamIndex,
		//dl_av_rescale_q(pos, AV_TIME_BASE_Q, _audioStream->time_base),
		0,
		AVSEEK_FLAG_ANY)<0)
		return false;

	_decodeBufOffset=0;
	if (_packet.data) {
		dl_av_free_packet(&_packet);
		_packet.data=0;
	}

	dl_avcodec_flush_buffers(_formatContext->streams[_audioStreamIndex]->codec);

	return true;
}

bool ffmpegBackend::openStream(AVFormatContext* fc, const int streamIndex) {

	if ((streamIndex<0) || (streamIndex>=fc->nb_streams))
		return false;

	AVCodecContext* dec=fc->streams[streamIndex]->codec;
	AVCodec* codec=dl_avcodec_find_decoder(dec->codec_id);
	if (!codec)
		return false;

	if (dl_avcodec_open(dec, codec)<0)
		return false;

	return true;
}

/*****************************************************************/

FXIMPLEMENT(ffmpegConfig, configFrame, 0, 0)

ffmpegConfig::ffmpegConfig(FXPacker* win) :
	configFrame(win, ffmpegBackend::name, CREDITS, LINK) {

	new FXLabel(matrix(), tr("No settings available"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
}

#endif
